==> odoo11_addon_sale_blanket_order-11.0.1.0.0.99.dev4-py2.py3-none-any.whl: odoo/addons/sale_blanket_order/wizard/create_sale_orders.py <==
# Copyright 2018 ACSONE SA/NV
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl).
from odoo import fields, models, api, _
from odoo.tools import float_is_zero
from collections import defaultdict
from odoo.exceptions import UserError


class BlanketOrderWizard(models.TransientModel):
    _name = 'sale.blanket.order.wizard'
    _description = 'Blanket Order Wizard'

    @api.model
    def _default_order(self):
        # in case the cron hasn't run
        self.env['sale.blanket.order'].expire_orders()
        if not self.env.context.get('active_id'):
            return False
        blanket_order = self.env['sale.blanket.order'].search(
            [('id', '=', self.env.context['active_id'])], limit=1)
        if blanket_order.state == 'expired':
            raise UserError(_('You can\'t create a sale order from '
                              'an expired blanket order!'))
        return blanket_order

    @api.model
    def _check_valid_blanket_order_line(self, bo_lines):
        precision = self.env['decimal.precision'].precision_get(
            'Product Unit of Measure')
        company_id = False

        for line in bo_lines:

            if line.order_id.state != 'open':
                raise UserError(
                    _('Sale Blanket Order %s is not open') %
                    line.order_id.name)

            if float_is_zero(
                    line.remaining_uom_qty, precision_digits=precision):
                raise UserError(
                    _('The sale has already been completed.'))

            line_company_id = line.company_id and line.company_id.id or False
            if company_id is not False \
                    and line_company_id != company_id:
                raise UserError(
                    _('You have to select lines '
                      'from the same company.'))
            else:
                company_id = line_company_id

    @api.model
    def _default_lines(self):
        blanket_order_line_obj = self.env['sale.blanket.order.line']
        blanket_order_line_ids = self.env.context.get('active_ids', False)
        active_model = self.env.context.get('active_model', False)

        if active_model == 'sale.blanket.order':
            bo_lines = self._default_order().line_ids
        else:
            bo_lines = blanket_order_line_obj.browse(blanket_order_line_ids)

        self._check_valid_blanket_order_line(bo_lines)

        lines = [(0, 0, {
            'blanket_line_id': l.id,
            'product_id': l.product_id.id,
            'date_schedule': l.date_schedule,
            'remaining_uom_qty': l.remaining_uom_qty,
            'price_unit': l.price_unit,
            'product_uom': l.product_uom,
            'qty': l.remaining_uom_qty,
            'partner_id': l.partner_id,
        }) for l in bo_lines]
        return lines

    blanket_order_id = fields.Many2one(
        'sale.blanket.order', readonly=True)
    sale_order_id = fields.Many2one(
        'sale.order',
        string='Purchase Order',
        domain=[('state', '=', 'draft')])
    line_ids = fields.One2many(
        'sale.blanket.order.wizard.line', 'wizard_id',
        string='Lines', default=_default_lines)

    @api.multi
    def create_sale_order(self):

        order_lines_by_customer = defaultdict(list)
        currency_id = 0
        pricelist_id = 0
        user_id = 0
        payment_term_id = 0
        for line in self.line_ids:
            if line.qty == 0.0:
                continue

            if line.qty > line.remaining_uom_qty:
                raise UserError(
                    _('You can\'t order more than the remaining quantities'))

            vals = {'product_id': line.product_id.id,
                    'name': line.product_id.name,
                    'product_uom': line.product_uom.id,
                    'sequence': line.blanket_line_id.sequence,
                    'price_unit': line.blanket_line_id.price_unit,
                    'blanket_order_line': line.blanket_line_id.id,
                    'product_uom_qty': line.qty,
                    'tax_id': [(6, 0, line.taxes_id.ids)]}
            order_lines_by_customer[line.partner_id.id].append((0, 0, vals))

            if currency_id == 0:
                currency_id = line.blanket_line_id.order_id.currency_id.id
            elif currency_id != line.blanket_line_id.order_id.currency_id.id:
                currency_id = False

            if pricelist_id == 0:
                pricelist_id = line.blanket_line_id.pricelist_id.id
            elif pricelist_id != line.blanket_line_id.pricelist_id.id:
                pricelist_id = False

            if user_id == 0:
                user_id = line.blanket_line_id.user_id.id
            elif user_id != line.blanket_line_id.user_id.id:
                user_id = False

            if payment_term_id == 0:
                payment_term_id = line.blanket_line_id.payment_term_id.id
            elif payment_term_id != line.blanket_line_id.payment_term_id.id:
                payment_term_id = False

        if not order_lines_by_customer:
            raise UserError(_('An order can\'t be empty'))

        if not currency_id:
            raise UserError(_('Can not create Sale Order from Blanket '
                              'Order lines with different currencies'))

        res = []
        for customer in order_lines_by_customer:
            order_vals = {
                'partner_id': customer,
            }
            if self.blanket_order_id:
                order_vals.update({
                    'origin': self.blanket_order_id.name,
                })
            order_vals.update({
                'user_id': user_id if user_id else False,
                'currency_id': currency_id if currency_id else False,
                'pricelist_id': pricelist_id if pricelist_id else False,
                'payment_term_id': (payment_term_id
                                    if payment_term_id
                                    else False),
                'order_line': order_lines_by_customer[customer],
            })
            sale_order = self.env['sale.order'].create(order_vals)
            res.append(sale_order.id)
        return {
            'domain': [('id', 'in', res)],
            'name': _('Sales Orders'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'sale.order',
            'view_id': False,
            'context': {'from_sale_order': True},
            'type': 'ir.actions.act_window'
        }


class BlanketOrderWizardLine(models.TransientModel):
    _name = 'sale.blanket.order.wizard.line'

    wizard_id = fields.Many2one('sale.blanket.order.wizard')
    blanket_line_id = fields.Many2one(
        'sale.blanket.order.line')
    product_id = fields.Many2one(
        'product.product',
        related='blanket_line_id.product_id',
        string='Product', readonly=True)
    product_uom = fields.Many2one(
        'product.uom',
        related='blanket_line_id.product_uom',
        string='Unit of Measure', readonly=True)
    date_schedule = fields.Date(
        related='blanket_line_id.date_schedule', readonly=True)
    remaining_uom_qty = fields.Float(
        related='blanket_line_id.remaining_uom_qty', readonly=True)
    qty = fields.Float(string='Quantity to Order', required=True)
    price_unit = fields.Float(
        related='blanket_line_id.price_unit', readonly=True)
    currency_id = fields.Many2one(
        'res.currency', related='blanket_line_id.currency_id'
    )
    partner_id = fields.Many2one(
        'res.partner',
        related='blanket_line_id.partner_id',
        string='Vendor', readonly=True,
    )
    taxes_id = fields.Many2many(
        'account.tax', related="blanket_line_id.taxes_id",
        readonly=True)

==> odoo11_addon_sale_blanket_order-11.0.1.0.2-py2.py3-none-any.whl: odoo/addons/sale_blanket_order/wizard/create_sale_orders.py <==
# Copyright 2018 ACSONE SA/NV
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl).
from odoo import fields, models, api, _
from odoo.tools import float_is_zero
from collections import defaultdict
from odoo.exceptions import UserError


class BlanketOrderWizard(models.TransientModel):
    _name = 'sale.blanket.order.wizard'
    _description = 'Blanket Order Wizard'

    @api.model
    def _default_order(self):
        # in case the cron hasn't run
        self.env['sale.blanket.order'].expire_orders()
        if not self.env.context.get('active_id'):
            return False
        blanket_order = self.env['sale.blanket.order'].search(
            [('id', '=', self.env.context['active_id'])], limit=1)
        if blanket_order.state == 'expired':
            raise UserError(_('You can\'t create a sale order from '
                              'an expired blanket order!'))
        return blanket_order

    @api.model
    def _check_valid_blanket_order_line(self, bo_lines):
        precision = self.env['decimal.precision'].precision_get(
            'Product Unit of Measure')
        company_id = False

        if all(float_is_zero(
                line.remaining_uom_qty, precision_digits=precision)
               for line in bo_lines):
            raise UserError(
                _('The sale has already been completed.'))

        for line in bo_lines:
            if line.order_id.state != 'open':
                raise UserError(
                    _('Sale Blanket Order %s is not open') %
                    line.order_id.name)
            line_company_id = line.company_id and line.company_id.id or False
            if company_id is not False \
                    and line_company_id != company_id:
                raise UserError(
                    _('You have to select lines '
                      'from the same company.'))
            else:
                company_id = line_company_id

    @api.model
    def _default_lines(self):
        blanket_order_line_obj = self.env['sale.blanket.order.line']
        blanket_order_line_ids = self.env.context.get('active_ids', False)
        active_model = self.env.context.get('active_model', False)

        if active_model == 'sale.blanket.order':
            bo_lines = self._default_order().line_ids
        else:
            bo_lines = blanket_order_line_obj.browse(blanket_order_line_ids)

        self._check_valid_blanket_order_line(bo_lines)

        lines = [(0, 0, {
            'blanket_line_id': l.id,
            'product_id': l.product_id.id,
            'date_schedule': l.date_schedule,
            'remaining_uom_qty': l.remaining_uom_qty,
            'price_unit': l.price_unit,
            'product_uom': l.product_uom,
            'qty': l.remaining_uom_qty,
            'partner_id': l.partner_id,
        }) for l in bo_lines.filtered(lambda l: l.remaining_uom_qty != 0.0)]
        return lines

    blanket_order_id = fields.Many2one(
        'sale.blanket.order', readonly=True)
    sale_order_id = fields.Many2one(
        'sale.order',
        string='Purchase Order',
        domain=[('state', '=', 'draft')])
    line_ids = fields.One2many(
        'sale.blanket.order.wizard.line', 'wizard_id',
        string='Lines', default=_default_lines)

    @api.multi
    def create_sale_order(self):

        order_lines_by_customer = defaultdict(list)
        currency_id = 0
        pricelist_id = 0
        user_id = 0
        payment_term_id = 0
        for line in self.line_ids:
            if line.qty == 0.0:
                continue

            if line.qty > line.remaining_uom_qty:
                raise UserError(
                    _('You can\'t order more than the remaining quantities'))

            vals = {'product_id': line.product_id.id,
                    'name': line.product_id.name,
                    'product_uom': line.product_uom.id,
                    'sequence': line.blanket_line_id.sequence,
                    'price_unit': line.blanket_line_id.price_unit,
                    'blanket_order_line': line.blanket_line_id.id,
                    'product_uom_qty': line.qty,
                    'tax_id': [(6, 0, line.taxes_id.ids)]}
            order_lines_by_customer[line.partner_id.id].append((0, 0, vals))

            if currency_id == 0:
                currency_id = line.blanket_line_id.order_id.currency_id.id
            elif currency_id != line.blanket_line_id.order_id.currency_id.id:
                currency_id = False

            if pricelist_id == 0:
                pricelist_id = line.blanket_line_id.pricelist_id.id
            elif pricelist_id != line.blanket_line_id.pricelist_id.id:
                pricelist_id = False

            if user_id == 0:
                user_id = line.blanket_line_id.user_id.id
            elif user_id != line.blanket_line_id.user_id.id:
                user_id = False

            if payment_term_id == 0:
                payment_term_id = line.blanket_line_id.payment_term_id.id
            elif payment_term_id != line.blanket_line_id.payment_term_id.id:
                payment_term_id = False

        if not order_lines_by_customer:
            raise UserError(_('An order can\'t be empty'))

        if not currency_id:
            raise UserError(_('Can not create Sale Order from Blanket '
                              'Order lines with different currencies'))

        res = []
        for customer in order_lines_by_customer:
            order_vals = {
                'partner_id': customer,
            }
            if self.blanket_order_id:
                order_vals.update({
                    'origin': self.blanket_order_id.name,
                })
            order_vals.update({
                'user_id': user_id if user_id else False,
                'currency_id': currency_id if currency_id else False,
                'pricelist_id': pricelist_id if pricelist_id else False,
                'payment_term_id': (payment_term_id
                                    if payment_term_id
                                    else False),
                'order_line': order_lines_by_customer[customer],
            })
            sale_order = self.env['sale.order'].create(order_vals)
            res.append(sale_order.id)
        return {
            'domain': [('id', 'in', res)],
            'name': _('Sales Orders'),
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'sale.order',
            'view_id': False,
            'context': {'from_sale_order': True},
            'type': 'ir.actions.act_window'
        }


class BlanketOrderWizardLine(models.TransientModel):
    _name = 'sale.blanket.order.wizard.line'

    wizard_id = fields.Many2one('sale.blanket.order.wizard')
    blanket_line_id = fields.Many2one(
        'sale.blanket.order.line')
    product_id = fields.Many2one(
        'product.product',
        related='blanket_line_id.product_id',
        string='Product', readonly=True)
    product_uom = fields.Many2one(
        'product.uom',
        related='blanket_line_id.product_uom',
        string='Unit of Measure', readonly=True)
    date_schedule = fields.Date(
        related='blanket_line_id.date_schedule', readonly=True)
    remaining_uom_qty = fields.Float(
        related='blanket_line_id.remaining_uom_qty', readonly=True)
    qty = fields.Float(string='Quantity to Order', required=True)
    price_unit = fields.Float(
        related='blanket_line_id.price_unit', readonly=True)
    currency_id = fields.Many2one(
        'res.currency', related='blanket_line_id.currency_id'
    )
    partner_id = fields.Many2one(
        'res.partner',
        related='blanket_line_id.partner_id',
        string='Vendor', readonly=True,
    )
    taxes_id = fields.Many2many(
        'account.tax', related="blanket_line_id.taxes_id",
        readonly=True)
